/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQSeries Workflow API Examples - Activity implemenation            */
/*                                                                    */
/* Example activity implementation program which can be used to       */
/* analyze unknown container structures and set member values. The    */
/* return code of the program is specified via a command line switch. */
/*                                                                    */
/* Maximum nesting depth is MAX_LEVEL_DEPTH                           */
/* Maximum element number is MAX_LEAF_COUNT                           */
/* Maximum length of a BINARY data member is MAX_BIN_LEN              */
/*                                                                    */
/**********************************************************************/

#define MAX_LEAF_COUNT  200
#define MAX_BIN_LEN     256
#define MAX_LEVEL_DEPTH  10

#include <fmctjinc.hxx>                 // compiler-dependent #includes
#include <fmcjprun.hxx>                 // MQWF C++ API interface
#include <ctype.h>                      // character classification
#include <time.h>                       // time functions


int main (int argc, char ** argv)
{

  FmcjReadWriteContainer                outctnr;

  vector<FmcjContainerElement>          members[MAX_LEVEL_DEPTH];
  vector<FmcjContainerElement>          leaves[MAX_LEAF_COUNT];
  FmcjContainerElement                  element;

  unsigned long                         ulMemberCount           = 0;

  char                                  membername[MAX_BIN_LEN];
  char                                  membertype[MAX_BIN_LEN];

  string                                container = "";

  string                                strvalbuf;
  char                                  binvalbuf[MAX_BIN_LEN];
  long                                  lvalbuf        = 0;
  double                                fvalbuf        = 0.0;

  string                                tInfo;
  unsigned long                         j, k, l        = 0;
  unsigned long                         index          = 0;
  time_t                                ltime          = 0;

  int                                   param           = -1;
  int                                   pimreturn       = 0;

  string                                setstr  ="";
  long                                  setlong = 0;
  double                                setdbl  = 0;

  FmcjExecutionService                  service;

  time(&ltime);

/*--------------------------------------------------------------------*/
/* Check command line parameters                                      */
/*--------------------------------------------------------------------*/

  if ( argc!=2 )
  {
    cout << "\nUsage: " << argv[0]
         << " <-r{0|1|2|3|4|5}>\n" << endl
         << "       -rx: x specifies the return code of the program\n\n"
         << "       Program output is written to fmctjpim.log.\n"
         << endl;
    return -1;
  }

  if (strncmp(argv[1],"-r0",3)==0) param=0;
  if (strncmp(argv[1],"-r1",3)==0) param=1;
  if (strncmp(argv[1],"-r2",3)==0) param=2;
  if (strncmp(argv[1],"-r3",3)==0) param=3;
  if (strncmp(argv[1],"-r4",3)==0) param=4;
  if (strncmp(argv[1],"-r5",3)==0) param=5;

  switch (param)
  {
   case -1:
     cout << "\nUsage: " << argv[0]
          << " <-r{0|1|2|3|4|5}>\n" << endl
          << "       -rx: x specifies the return code of the program\n\n"
          << "       Program output is written to fmctjpim.log.\n"
          << endl;
     return -1;
     break;
   case 0: pimreturn = 0; break;
   case 1: pimreturn = 1; break;
   case 2: pimreturn = 2; break;
   case 3: pimreturn = 3; break;
   case 4: pimreturn = 4; break;
   case 5: pimreturn = 5; break;
  }

  ofstream file1("fmctjpim.log");

  file1 << endl << endl
        << "-------------------------------------------------------" << endl
        << "API Tutorial - Activity implementation example program. " << endl
        << "Called at : " << ctime(&ltime) << endl
        << "Desired RC: " << pimreturn << "\n"
        << endl;


/*-------------------------------------------------------------------*/
/* Init API environment, allocate result object.                     */
/*-------------------------------------------------------------------*/

  APIRET rc = FmcjGlobal::Connect();
  FmcjResult * result = FmcjResult::ObjectOfCurrentThread();

/*-------------------------------------------------------------------*/
/* Establish a Passthrough session to the specified system and       */
/* system group. Note: Passthrough is not absolutely necessary to    */
/* handle the input and output containers only. In this sample, it   */
/* is merely used to add information about the logged-on user to the */
/* log file.                                                         */
/*-------------------------------------------------------------------*/

  // only establish a passthrough session

  rc = service.Passthrough();
  if (rc != FMC_OK)
  {
    file1 << "Establishing Passthrough failed with RC "
          << rc << " - " << result->MessageText() << endl;
    cout  << "Establishing Passthrough failed with RC "
          << rc << " - " << result->MessageText() << endl;
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }

  file1 << "Logged on with UserID '" << service.UserID()
        << "' on system '" << service.SystemName()
        << "' at system group '" << service.SystemGroupName()
        << "'." << endl << "Timeout value: " << service.Timeout() << endl;
  cout  << "Logged on with UserID '" << service.UserID()
        << "' on system '" << service.SystemName()
        << "' at system group '" << service.SystemGroupName()
        << "'." << endl << "Timeout value: " << service.Timeout() << endl;


/************************************************************************/
/* First we cope with the activity's INPUT container:                   */
/*  - get the container                                                 */
/*  - get the leaves of the container                                   */
/*  - display the member values                                         */
/************************************************************************/

/*----- Get Input Container --------------------------------------------*/

  FmcjReadOnlyContainer                 inctnr;

  rc = FmcjContainer::InContainer(inctnr);
  if (rc != FMC_OK)
  {
    file1 << "Getting input container failed with RC "
          << rc << " - " << result->MessageText() << endl;
    cout  << "Getting input container failed with RC "
          << rc << " - " << result->MessageText() << endl;
    service.Logoff();
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -2;
  }

  file1 << endl << "Received input container '" << inctnr.Type() << "' with RC " << rc << endl << endl;
  cout  << endl << "Received input container '" << inctnr.Type() << "' with RC " << rc << endl;

/*----- Get Leave Count ------------------------------------------------*/

  unsigned long ulLeafCount = inctnr.AllLeafCount();

  file1 << "Input container AllLeafCount() =  " << ulLeafCount << endl;

/*----- Get the Leaves -------------------------------------------------*/

  inctnr.AllLeaves(leaves[0]);

  if (result->Rc() != FMC_OK)
  {
    file1 << "FmcjContainerAllLeaves failed with RC "
          << result->Rc() << endl;
    cout  << "FmcjContainerAllLeaves failed with RC "
          << result->Rc() << endl;
  }

  if (ulLeafCount != leaves[0].size())
  {
    file1 << "LeafCount - vector size mismatch: "
          << "ulLeafCount = " << ulLeafCount
          << " size = "       << leaves[0].size()
          << endl;
  }

/*----- Show the data members ------------------------------------------*/

  file1 << "Input container leaves:" << endl;

  for (j=0; j < ulLeafCount; j++)
  {
    element = leaves[0][j];
    strcpy(membername, element.FullName().c_str());
    strcpy(membertype, element.Type().c_str());

    if (strcmp(membertype, "STRING") == 0)
    {
      rc = element.Value(strvalbuf);
      if ( (rc != FMC_OK) && (rc != FMC_ERROR_MEMBER_NOT_SET) )
      {
        file1 << "Failed to access " << membername << " with RC " << rc << endl;
      }
      else
      {
        if (rc==FMC_ERROR_MEMBER_NOT_SET)
        {
          file1 << "STRING '" << membername  << "'= <not set>" << endl;
        }
        else
        {
          file1 << "STRING '" << membername << "'= " << strvalbuf  << endl;
        }
      }
    } /* end if STRING */

    if (strcmp(membertype, "LONG") == 0)
    {
      rc = element.Value(lvalbuf);
      if ( (rc != FMC_OK) && (rc != FMC_ERROR_MEMBER_NOT_SET) )
      {
        file1 << "Failed to access " << membername << " with RC " << rc << endl;
      }
      else
      {
        if (rc==FMC_ERROR_MEMBER_NOT_SET)
        {
          file1 << "LONG   '" << membername  << "'= <not set>" << endl;
        }
        else
        {
          file1 << "LONG   '" << membername << "'= " << lvalbuf  << endl;
        }
      }
    } /* end if LONG */

    if (strcmp(membertype, "FLOAT") == 0)
    {
      rc = element.Value(fvalbuf);
      if ( (rc != FMC_OK) && (rc != FMC_ERROR_MEMBER_NOT_SET) )
      {
        file1 << "Failed to access " << membername << " with RC " << rc << endl;
      }
      else
      {
        if (rc==FMC_ERROR_MEMBER_NOT_SET)
        {
          file1 << "FLOAT  '" << membername  << "'= <not set>" << endl;
        }
        else
        {
          file1 << "FLOAT  '" << membername << "'= " << fvalbuf  << endl;
        }
      }
    } /* end if FLOAT */

    if (strcmp(membertype, "BINARY") == 0)
    {
      rc = element.Value(&binvalbuf, MAX_BIN_LEN);
      if ( (rc != FMC_OK) && (rc != FMC_ERROR_MEMBER_NOT_SET) )
      {
        file1 << "Failed to access " << membername << " with RC " << rc << endl;
      }
      else
      {
        // Binary values may contain 0x00 or control characters, hence
        // Do not try to print them!

        if (rc==FMC_ERROR_MEMBER_NOT_SET)
        {
          file1 << "BINARY '" << membername  << "'= <not set>" << endl;
        }
        else
        {
          file1 << "BINARY '" << membername  << "'= <exists, but is not displayed>" << endl;
        }
      }
     } /* end if BINARY */

  } /* end for */


/************************************************************************/
/* Now we process the activity's OUTPUT container, but only if the      */
/* program was not started with the -s switch                           */
/*  - get the container                                                 */
/*  - navigate through the ContainerElement levels to the leaves        */
/*  - modify the member values                                          */
/*  - send the container to the server                                  */
/************************************************************************/

/*----- Get Output Container -------------------------------------------*/

  rc= FmcjContainer::OutContainer(outctnr);
  if (rc != FMC_OK)
  {
    file1 << "Getting output container failed with RC " << rc << " - "
          << result->MessageText() << endl;
    cout  << "Getting output container failed with RC " << rc << " - "
          << result->MessageText() << endl;
    service.Logoff();
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -3;
  }

  file1 << endl << "Received output container '" << outctnr.Type() << "' with RC " << rc << endl;
  cout  << "Received output container '" << outctnr.Type() << "' with RC " << rc << endl;


/*----- Navigate through the ContainerElement structures ---------------*/

  unsigned long level            = 0;  // Current nesting level
  unsigned long start            = 0;  // Start "stack pointer" of vectors
  unsigned long current          = 0;  // End "stack pointer" of vectors
  bool AllLeavesReached = false;

  // Get the number of 1st level container elements

  ulMemberCount = outctnr.MemberCount();

  if (ulMemberCount == 0)    	// No data structure at all, error?
  {
    AllLeavesReached = true;
  }
  else				// Get the vector of 1st level container elements
  {
    outctnr.StructMembers(members[level]);
  }

  while (AllLeavesReached == false)
  {
    file1 << endl << "Number of members: " << ulMemberCount << endl;

    // Check consistency

    if (members[level].size() != ulMemberCount)
    {
      file1 << endl << "MemberCount - Vector Size mismatch: "
            << "ulMemberCount = " << ulMemberCount
            << " Size = "         << members[level].size() << endl;
    }

    element = members[level][0];

    file1 << "IsStruct-IsLeaf-IsArray-Cardinality-Membername" << endl;

    for (j=0; j < ulMemberCount; j++)
    {
      file1 << "       "
            << element.IsStruct();

      if (element.IsStruct())
      {
        // Put the next nesting level of this data structure "on the stack"
        leaves[current].erase(leaves[current].begin(),leaves[current].end());
        element.StructMembers(leaves[current]);
        current++;
      }

      file1 << "      " << element.IsLeaf() << "       " << element.IsArray()
            << "           " << element.Cardinality() << " "
            << element.FullName() << endl;

      element = members[level][j];

    } /* end for */

    if (start >= current)        // Nothing "on the stack"
    {
      AllLeavesReached = true;
    }
    else
    {
      level++;
      // Get the next vector from the stack and increase stack pointer
      members[level] = leaves[start];
      start++;
      ulMemberCount = members[level].size();
    }

  } /* end while */

/*----- Modify the data members ----------------------------------------*/

  file1 << endl << endl << "Setting the data members... " << endl;
  cout  << endl << "Setting the data members... " << endl;

  leaves[0].erase(leaves[0].begin(),leaves[0].end());
  outctnr.Leaves(leaves[0]);
  if (result->Rc() != FMC_OK)
  {
    file1 << "FmcjContainerLeaves failed with RC " << result->Rc() << endl;
    cout  << "FmcjContainerLeaves failed with RC " << result->Rc() << endl;
  }

  ulLeafCount = leaves[0].size();
  element = leaves[0][0];

  for (j=1; j <= ulLeafCount; j++)
  {
    strcpy(membername, element.FullName().c_str());
    strcpy(membertype, element.Type().c_str());

    if (strcmp(membertype, "STRING") == 0)
    {
      if (   membername[strlen(membername)-1]==']'
          || membername[strlen(membername)-1]==')')

      // Array element, take the array function
      {
	// Remember the index and remove the suffix from the name

        if (membername[strlen(membername)-1]==']')
        {
          memset(&membername[strlen(membername)-1], 0, 1);
          index = atoi( &(strrchr(membername, '[')[1]) );
          memset(&(strrchr(membername, '[')[0]), 0x00, 1);
        }
        else
        {
          memset(&membername[strlen(membername)-1], 0, 1);
          index = atoi( &(strrchr(membername, '(')[1]) );
          memset(&(strrchr(membername, '(')[0]), 0x00, 1);
        }

        cout << "Enter a value for STRING array member '"
             << membername << "[" << index << "]'? [y/n] "; cout.flush();

        cin >> setstr;

        if (strncmp(setstr.c_str(),"y",1)==0)
        {
          cout << "Value: "; cout.flush();
          cin >> setstr;

          rc= outctnr.SetValue(string(membername),index,setstr);
          if (rc != FMC_OK)
          {
            file1 << endl << "Setting ArrayStringValue failed with RC " << rc << endl;
            cout  << endl << "Setting ArrayStringValue failed with RC " << rc << endl;
          }
          rc= outctnr.Value(membername,index,strvalbuf);

          file1 << "Setting ArrayStringValue '" << membername << "[" << index << "]' = " << strvalbuf
                << endl;
          cout  << "Setting ArrayStringValue '" << membername << "[" << index << "]' = " << strvalbuf
                << endl;
        }
      }
      else
      {

        cout << "Enter a value for STRING member '"
             << membername << "'? [y/n] "; cout.flush();
        cin >> setstr;

	if (strncmp(setstr.c_str(),"y",1)==0)
	{
          cout << "Value: "; cout.flush();
          cin >> setstr;
		
          rc= outctnr.SetValue(membername,setstr);
          if (rc != FMC_OK)
          {
            file1 << endl << "Setting StringValue failed with RC " << rc << endl;
            cout  << endl << "Setting StringValue failed with RC " << rc << endl;
          }
          rc= outctnr.Value(membername,strvalbuf);

          file1 << "Setting StringValue '" << membername << "'= " << strvalbuf << endl;
          cout  << "Setting StringValue '" << membername << "'= " << strvalbuf << endl;
        }
      }
    } /* end if STRING */


    if (strcmp(membertype, "LONG") == 0)
    {
      if (   membername[strlen(membername)-1]==']'
          || membername[strlen(membername)-1]==')')

      // Array element, take the array function
      {
	// Remember the index and remove the suffix from the name

        if (membername[strlen(membername)-1]==']')
        {
          memset(&membername[strlen(membername)-1], 0, 1);
          index = atoi( &(strrchr(membername, '[')[1]) );
          memset(&(strrchr(membername, '[')[0]), 0x00, 1);
        }
        else
        {
          memset(&membername[strlen(membername)-1], 0, 1);
          index = atoi( &(strrchr(membername, '(')[1]) );
          memset(&(strrchr(membername, '(')[0]), 0x00, 1);
        }

        cout << "Enter a value for LONG array member '"
             << membername << "[" << index << "]'? [y/n] "; cout.flush();

        cin >> setstr;

        if (strncmp(setstr.c_str(),"y",1)==0)
        {
      	  do
      	  {
      	    cout << "Value: ";
      	    cout.flush();
      	    cin >> setstr;
      	    l=0;
      	    for (k=0; k<strlen(setstr.c_str()); k++)
      	    { if (isdigit(setstr.c_str()[k])) l++; }
      	  } while (k!=l);
      	  setlong = atoi(setstr.c_str());

          rc= outctnr.SetValue(membername,index,setlong);
          if (rc != FMC_OK)
          {
            file1 << endl << "Setting ArrayLongValue failed with RC " << rc << endl;
            cout  << endl << "Setting ArrayLongValue failed with RC " << rc << endl;
          }
          rc= outctnr.Value(membername,index,lvalbuf);

          file1 << "Setting ArrayLongValue '" << membername << "["
                << index << "]' = " << lvalbuf << endl;
          cout  << "Setting ArrayLongValue '" << membername << "["
                << index << "]' = " << lvalbuf << endl;
         }
      }
      else
      {

        cout << "Enter a value for LONG member '"
             << membername << "'? [y/n] "; cout.flush();

        cin >> setstr;

        if (strncmp(setstr.c_str(),"y",1)==0)
	{
      	  do
      	  {
      	    cout << "Value: ";
      	    cout.flush();
      	    cin >> setstr;
      	    l=0;
      	    for (k=0; k<strlen(setstr.c_str()); k++)
      	    { if (isdigit(setstr.c_str()[k])) l++; }
      	  } while (k!=l);
      	  setlong = atoi(setstr.c_str());

          rc= outctnr.SetValue(membername,setlong);
          if (rc != FMC_OK)
          {
            file1 << "\nSetting LongValue failed with RC " << rc << endl;
            cout  << "\nSetting LongValue failed with RC " << rc << endl;
          }
          rc= outctnr.Value(membername,lvalbuf);

          file1 << "Setting LongValue '" << membername << "' = " << lvalbuf << endl;
          cout  << "Setting LongValue '" << membername << "' = " << lvalbuf << endl;
        }
      }
    } /* end if LONG */


    if (strcmp(membertype, "FLOAT") == 0)
    {
      if (   membername[strlen(membername)-1]==']'
          || membername[strlen(membername)-1]==')')

      // Array element, take the array function
      {
	// Remember the index and remove the suffix from the name

        if (membername[strlen(membername)-1]==']')
        {
          memset(&membername[strlen(membername)-1], 0, 1);
          index = atoi( &(strrchr(membername, '[')[1]) );
          memset(&(strrchr(membername, '[')[0]), 0x00, 1);
        }
        else
        {
          memset(&membername[strlen(membername)-1], 0, 1);
          index = atoi( &(strrchr(membername, '(')[1]) );
          memset(&(strrchr(membername, '(')[0]), 0x00, 1);
        }

        cout << "Enter a value for FLOAT array member '"
             << membername << "[" << index << "]'? [y/n] "; cout.flush();

        cin >> setstr;

        if (strncmp(setstr.c_str(),"y",1)==0)
	{
          cout << "Value: "; cout.flush();
          cin >> setdbl;

          rc= outctnr.SetValue(membername,index,setdbl);
          if (rc != FMC_OK)
          {
            file1 << "\nSetting ArrayFloatValue failed with RC " << rc << endl;
            cout  << "\nSetting ArrayFloatValue failed with RC " << rc << endl;
          }
          rc= outctnr.Value(membername,index,fvalbuf);

          file1 << "Setting ArrayFloatValue  " << membername << "[" << index << "] = " << fvalbuf
                << endl;
          cout  << "Setting ArrayFloatValue  " << membername << "[" << index << "] = " << fvalbuf
                << endl;
        }
      }
      else
      {
        cout << "Enter a value for FLOAT member '"
             << membername << "'? [y/n] "; cout.flush();

        cin >> setstr;

        if (strncmp(setstr.c_str(),"y",1)==0)
	{
          cout << "Value: "; cout.flush();
          cin >> setdbl;

          rc= outctnr.SetValue(membername,setdbl);
          if (rc != FMC_OK)
          {
            file1 << "\nSetting FloatValue failed with RC " << rc
                  << endl;
            cout  << "\nSetting FloatValue failed with RC " << rc
                  << endl;
          }
          rc= outctnr.Value(membername,fvalbuf);

          file1 << "Setting FloatValue '" << membername << "' = " << fvalbuf
                << endl;
          cout  << "Setting FloatValue '" << membername << "' = " << fvalbuf
                << endl;
        }
      }
    } /* end if FLOAT */


    if (strcmp(membertype, "BINARY") == 0)
    {
      if (   membername[strlen(membername)-1]==']'
          || membername[strlen(membername)-1]==')')

      // Array element, take the array function
      {
	// Remember the index and remove the suffix from the name

        if (membername[strlen(membername)-1]==']')
        {
          memset(&membername[strlen(membername)-1], 0, 1);
          index = atoi( &(strrchr(membername, '[')[1]) );
          memset(&(strrchr(membername, '[')[0]), 0x00, 1);
        }
        else
        {
          memset(&membername[strlen(membername)-1], 0, 1);
          index = atoi( &(strrchr(membername, '(')[1]) );
          memset(&(strrchr(membername, '(')[0]), 0x00, 1);
        }

        rc= outctnr.SetValue(membername,index,"<Binary test>", 13);
        if (rc != FMC_OK)
        {
          file1 << "Setting ArrayBinaryValue failed with RC " << rc << endl;
          cout  << "Setting ArrayBinaryValue failed with RC " << rc << endl;
        }
        rc= outctnr.Value(membername,index,binvalbuf,13);

        file1 << "Setting ArrayBinaryValue '" << membername << "[" << index << "]' = " << "<Binary test>"
              << endl;
        cout  << "Setting ArrayBinaryValue '" << membername << "[" << index << "]' = " << "<Binary test>"
              << endl;
      }
      else
      {
        rc= outctnr.SetValue(membername,"<Binary test>", 13);
        if (rc != FMC_OK)
        {
          file1 << "Setting BinaryValue failed with RC " << rc << endl;
          cout  << "Setting BinaryValue failed with RC " << rc << endl;
        }
        rc= outctnr.Value(membername,binvalbuf,13);

        file1 << "Setting BinaryValue      " << membername << " = " << "<Binary test>"
              << endl;
        cout  << "Setting BinaryValue      " << membername << " = " << "<Binary test>"
              << endl;
      }
    } /* end if BINARY */

    element = leaves[0][j];

  } /* end for */


/*-------------------------------------------------------------------*/
/* Send output container to PEA                                      */
/*-------------------------------------------------------------------*/
  rc = FmcjContainer::SetOutContainer(outctnr);

  file1 << "\nSet Output Container - rc   =  " << rc << endl;
  cout  << "\nSet Output Container - rc   =  " << rc << endl;


/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/

  cout << "\nLogging off." << endl;
  rc = service.Logoff();
  cout << "Deinitializing API environment." << endl;
  rc = FmcjGlobal::Disconnect();
  file1 << endl << "Exiting with RC " << pimreturn << "." << endl;
  cout  << endl << "Exiting with RC " << pimreturn << "." << endl;
  delete(result);
  return pimreturn;
}
