/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQSeries Workflow API Examples - Handling process templates        */
/*                                                                    */
/* This sample program shows how to start process instances from an   */
/* existing process template.                                         */
/* Additionally, it demonstrates the differences between primary and  */
/* secondary object properties.                                       */
/**********************************************************************/

// Import necessary MQWF Java API Interface

import com.ibm.workflow.api.*;
import com.ibm.workflow.api.ServicePackage.*;

// Various needed classes

import java.io.*;


public class ProcessTemplates {

public static void main (String[] args)
{

  ProcessTemplate[] tList;

  int i = 0;
  int j = 0;
  int threshold = 10;

  ProcessInstance        processInstance;

  String choice ="";
  String PTListSort ="NAME ASC";
  int   usrchoice = 0;

  ReadWriteContainer iictnr;

  BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
  boolean again = false;


/*--------------------------------------------------------------------*/
/* Check command line parameters, logon to system                     */
/*--------------------------------------------------------------------*/

  if (args.length!=4)
  {
    System.out.println("\nUsage: java ProcessTemplates <name of agent> <loc|jndi> <UserID> <Password>");
    System.exit(-1);
  }

  try { // general try clause for whole example

  Agent agent = new Agent();
  if      (args[1].equalsIgnoreCase("loc"))  agent.setLocator(Agent.LOC_LOCATOR);
  else if (args[1].equalsIgnoreCase("jndi")) agent.setLocator(Agent.JNDI_LOCATOR);
  else { System.out.println("Invalid locator policy: " + args[1]);
         System.exit(-1); }
  agent.setName(args[0]);
  ExecutionService service = agent.locate("","");
  System.out.println();
  service.logon2( args[2], args[3], SessionMode.DEFAULT, AbsenceIndicator.LEAVE );
  System.out.println("Logged on with UserID '" + service.userID() +
                     "' on system '" + service.systemName() +
                     "' at system group '" + service.systemGroupName() + "'.");


/*--------------------------------------------------------------------*/
/* Display current template list, query user for action               */
/*--------------------------------------------------------------------*/

  do                                    // Beginning of main tool loop
  {

    // Display as many process templates as specified with threshold

    System.out.println("\nAvailable process templates:\n");

    tList = service.queryProcessTemplates( "", PTListSort, new Integer(threshold) );

    System.out.println("    Input container\n"
                       + "#   needed?           Template name\n"
                       + "-----------------------------------");

    if (tList.length > 0 )
    {
      for( i=0; i< tList.length; i++ )
      {
        System.out.print(i);
        if (i<10) System.out.print(" ");
        if (tList[i].inContainerNeeded()) System.out.print("  yes               ");
        else System.out.print("  no                ");
        System.out.println(tList[i].name());
      }
    }
    else System.out.println("  <no templates available>");

    System.out.print("\nCommands: [v] View template info                    [c] Create an instance\n"
         + "          [s] Create and start an instance          [q] Quit program\n"
         + "[v/s/c/q]? ");

    choice = in.readLine(); System.out.println();

    usrchoice=0;

    if (choice.equalsIgnoreCase("v")) usrchoice=1;
    if (choice.equalsIgnoreCase("s")) usrchoice=2;
    if (choice.equalsIgnoreCase("c")) usrchoice=3;
    if (choice.equalsIgnoreCase("q")) usrchoice=255;

    if (tList.length>0)

    switch (usrchoice)
    {

      case 1:
/*-------------------------------------------------------------------*/
/* Demonstrates the difference between primary and secondary object  */
/* properties of process templates. At first, only the primary       */
/* properties are filled with content, the secondary properties need */
/* a Refresh() to be accessible.                                     */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter template number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(tList.length-1) && tList.length>0)
        {
          System.out.println("Please use range 0-" + Integer.toString(tList.length-1) + ".");
        }
        else                                   // User's choice is in range
        {
          System.out.println("Template properties before Refresh():");
          System.out.println("-------------------------------------");;
          System.out.println("   (P) Name     : " + tList[i].name());
          System.out.println("   (P) Category : " + tList[i].category());
          System.out.println("   (P) Object id: " + tList[i].persistentOid() + "\n");
          System.out.println("   (S) Process administrator: " + tList[i].processAdmin());
          System.out.println("   (S) Input container name : " + tList[i].inContainerName());
          System.out.println("   (S) Output container name: " + tList[i].outContainerName() + "\n");

          // Now fetching secondary template properties...

          tList[i].refresh();

          System.out.println("Template properties after Refresh():");
          System.out.println("------------------------------------");;
          System.out.println("   (P) Name     : " + tList[i].name());
          System.out.println("   (P) Category : " + tList[i].category());
          System.out.println("   (P) Object id: " + tList[i].persistentOid() + "\n");
          System.out.println("   (S) Process administrator: " + tList[i].processAdmin());
          System.out.println("   (S) Input container name : " + tList[i].inContainerName());
          System.out.println("   (S) Output container name: " + tList[i].outContainerName() + "\n");
   } /* end else */

        break;


      case 2:
/*-------------------------------------------------------------------*/
/* Demonstrates how to create and start a process instance.          */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter template number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(tList.length-1) && tList.length>0)
        {
          System.out.println("Please use range 0-" + Integer.toString(tList.length-1) + ".");
        }
        else                                   // User's choice is in range
        {
          if (tList[i].inContainerNeeded())
          {
            // Query for input container values only for "CreditRequest"

            if (tList[i].name().equals("CreditRequest"))
            {
              iictnr = tList[i].initialInContainer();

         System.out.print("FirstName: ");

         choice = in.readLine();

         iictnr.setString("FirstName",choice);

         System.out.print("LastName: ");

         choice = in.readLine();

         iictnr.setString("LastName",choice);

              // End of query. Now creating and starting instance with the
              // filled initial input container.

              processInstance = tList[i].createAndStartInstance2( "", "", "", iictnr, false );
       }
       else
       {
         // Create and start instance without passing an input container
              processInstance = tList[i].createAndStartInstance( "", "", "", false );
            }

          }
          else
          {
       // Create and start instance without passing an input container
       processInstance = tList[i].createAndStartInstance( "", "", "", false );
          }

    } // end else: user's choice is in range

        break;


      case 3:
/*-------------------------------------------------------------------*/
/* Demonstrates how to create a process instance                     */
/*-------------------------------------------------------------------*/

   do
   {
     again=false;
          System.out.print("Please enter template number: ");
     choice = in.readLine();
     try { i = Integer.parseInt(choice); }
     catch (NumberFormatException e) { again=true; }
        } while (again);

   System.out.println();

        if (i>(tList.length-1) && tList.length>0)
        {
          System.out.println("Please use range 0-" + Integer.toString(tList.length-1) + ".");
        }
        else                                   // User's choice is in range
        {
     // Create an instance without starting it
          processInstance = tList[i].createInstance( "", "", "", false );
        }

 break;

 }                                      // end of switch
 else System.out.println("\nNo templates available!");

 } while ( (usrchoice!=255) );          // end of main tool loop


/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/

  System.out.println("\nLogging off.");
  service.logoff();
  System.exit(0);

  } // end of general try clause

  catch(FmcException e)
  {
    // Catch and report details about the FmcException
    System.out.println("FmcException occured");
    System.out.println("  RC         : " + e.rc);
    System.out.println("  Origin     : " + e.origin);
    System.out.println("  MessageText: " + e.messageText);
    System.out.println("  Exception  : " + e.getMessage());
    System.out.println("  Parameters : ");
    for ( i = 0; i < e.parameters.length ; i++)
    {
      System.out.println("    " + e.parameters[i] );
    }
    System.out.println("  StackTrace : ");
    e.printStackTrace();
  }
  catch(Exception e)
  {
    // Catch and report any exception that occurred.
    e.printStackTrace();
  }

} // end of void main

} // end of ProcessTemplates

